import argparse
from pathlib import Path


parser = argparse.ArgumentParser()
subparsers = parser.add_subparsers(dest='command', required=True)

# `render`: Render Marquedown documents

render = subparsers.add_parser('render', help='Render Markdown and Marquedown documents')
render.add_argument('-i', '--source',
    dest='source',
    help='Source root directory of Marquedown documents',
    type=Path,
    required=True,
)
render.add_argument('-o', '--destination',
    dest='destination',
    help='Destination root directory for rendered and templated documents',
    type=Path,
    required=True,
)
render.add_argument('-t', '--template',
    dest='template',
    help='Template HTML file. Must contain `{document}` where to insert the rendered document.',
    type=Path,
    default=None,
)




if __name__ == '__main__':
    args = parser.parse_args()
    
    # `render`: Render Marquedown documents
    if args.command == 'render':
        from .commands.render import render_documents
        render_documents(
            args.source,
            args.destination,
            args.template,
        )


import re


RE_BBLIKE = re.compile(r'\[(\/*)([\w\d\.]*)(?:[ ]*\#([a-zA-Z][\w\d\-\_\:\.]*))?\](?!\(.*\))')


def _replace_segment(text: str, i: int, j: int, replacement: str):
    """Remove segment of text and insert a replacement."""
    return text[:i] + replacement + text[j:]


def bblike(document: str):
    """Notation for BBCode-like HTML tags.
    
    Marquedown:
        Hello, [.green]there[/]!
        How [b]are [i]you?[//]

    HTML:
        Hello, <div class="green">there</div>!
        How <b>are <i>you?</i></b>
    """

    # Keep track of opened tags
    stack = []

    # Offset is used to correct for the lengths
    # of previous replacements when replacing new matches
    offset = 0

    for m in list(RE_BBLIKE.finditer(document)):
        escapers = len(m.group(1))
        tag, *classes = m.group(2).split('.')
        id_ = m.group(3)

        if escapers > 0:
            elements = []
            
            while stack and escapers:
                last_tag = stack.pop()

                if tag == "" or last_tag == tag:
                    escapers -= 1

                # Generate HTML element and add to be appended
                elements.append(f'</{last_tag}>')

            # Insert elements at match
            replacement = "".join(elements)
            document = _replace_segment(document, m.start() + offset, m.end() + offset, replacement)

            # Adjust offset
            offset += len(replacement) - len(m.group(0))
        
        else:
            # Default tag is `div` for convenience
            if not tag:
                tag = 'div'
            
            # Generate HTML element
            element = f'<{tag}'

            if classes:
                element += f' class="{" ".join(classes)}"'

            if id_ is not None:
                element += f' id="{id_}"'

            element += '>'

            # Insert tag at match
            document = _replace_segment(document, m.start() + offset, m.end() + offset, element)

            # Adjust offset
            offset += len(element) - len(m.group(0))

            # Add as opened tag
            stack.append(tag)

    return document


from __future__ import annotations

import re
import qrcode
from pathlib import Path
from itertools import count


RE_QR_INLINE = re.compile(r'\!\[qr\:(?P<alt>[^\n\s]+)\]\((?P<text>[^\n\s]+)\)')
RE_QR_MULTILINE = re.compile(r'\[o\] (?P<alt>.+)(?: \.*\[o\])?(?P<text>(?:\n\| .+)+)\n\[o\]$', re.MULTILINE)


class QRGenerator:
    """
    Generate QR codes

    Every image gets a unique index and it is increment for each.
    """

    def __init__(self, output_dir: Path, reference_dir: Path):
        self.output_dir = Path(output_dir)
        self.reference_dir = Path(reference_dir)

    def generate(self, text: str, alt: str) -> str:
        """
        Generate QR code from text.
        Saves image to file and returns HTML referencing the image.
        """

        # Get available filename
        image_name = self.get_available_filename(alt)
        
        # Generate and save image
        image = qrcode.make(text)
        self.output_dir.mkdir(parents=True, exist_ok=True)
        image.save(self.output_dir / image_name)

        # Reference image and return HTML
        image_path = self.reference_dir / image_name
        return f'<img class="qr" src="{image_path!s}" alt="{alt}">'

    def get_available_filename(self, alt: str) -> str:
        """Increment an index until an available filename is found."""

        # Make alt valid for filename
        alt = re.sub(r'[^\w\d\-]+', '-', alt.lower())

        image_name = f'qr-{alt}.png'
        image_path = self.output_dir / image_name

        for i in count():
            if not image_path.exists():
                return image_name

            image_name = f'qr-{alt}-{i}.png'
            image_path = self.output_dir / image_name

    def repl_qr(self, match: re.Match) -> str:
        text, alt = match.group('text', 'alt')

        # Remove preceding pipelines at the beginning of lines
        text = " ".join([line.lstrip('| ') for line in text.split('\n')])
        # Replace `\n` in text with actual line breaks
        text = re.sub(r'\\n\s*', '\n', text)

        return self.generate(text, alt)


def qr(document: str, qrgen: QRGenerator) -> str:
    """
    Notation for QR codes

    QR code images will be generated and saved,
    and image elements inserted referencing their file locations.

    Example:
        Marquedown:
            ![qr:monero-wallet](monero:abcdefghijklmnopqrstuvwxyz)

            [o] Bee Movie transcript ..................[o]
            | According to all known laws of aviation,
            | there is no way that a bee should be able
            | to fly. Its wings are too small to get its
            | fat little body off the ground. The bee,
            | of course, flies anyway because bees don't
            | care what humans think is impossible.
            [o]

        HTML:
            <img src="qr/qr-monero-wallet.png" alt="monero-wallet">

            <img class="qr" src="qr/qr-bee-movie-transcript.png" alt="Bee Movie transcript">
    """

    document = RE_QR_MULTILINE.sub(qrgen.repl_qr, document)
    document = RE_QR_INLINE.sub(qrgen.repl_qr, document)

    return document


__version__ = '0.10.3'

import markdown as md


def marquedown(document: str, **kwargs):
    """Convert both Marquedown and Markdown into HTML."""

    if kwargs.get('citation', True):
        from .citation import citation
        document = citation(document)

    if kwargs.get('labellist', True):
        from .labellist import labellist
        document = labellist(document)

    if kwargs.get('qrgen') is not None:
        # Parse QR codes with provided QRGenerator
        from .qr import qr
        document = qr(document, kwargs.get('qrgen'))

    if kwargs.get('video', True):
        from .video import video
        document = video(document)
        
    html = md.markdown(document)

    if kwargs.get('bblike', True):
        from .bblike import bblike
        html = bblike(html)

    return html




def tagstrip(text: str, tag: str):
    """Strips HTML tag from sides of text."""
    
    if contained(text, tag):
        left, right = tags(tag)
        return text.strip()[len(left):-len(right)].strip(" ").strip('\n')

    return text


def contained(text: str, tag: str):
    """Returns whether text is contained within a certain HTML tag."""
    left, right = tags(tag)
    stripped_text = text.strip()
    return stripped_text.startswith(left) and stripped_text.endswith(right)


def tags(tag: str):
    """Get left-hand and right-hand sides of a tag."""
    return f'<{tag}>', f'</{tag}>'